<?php defined('SYSPATH') or die('No direct script access.');

class Tree
{
    protected $table = '';
    private $parent=0,$children,$data;

    public function __construct($item=1)
    {
	$this->children = array();
	$this->set($item,is_numeric($item)?true:false);
    }

    public function __get($value)
    {
	return $this->data[$value];
    }

    public function __set($property, $value)
    {
	$this->data[$property]=$value;
    }

    public function hasDescendants()
    { 
	return (count($this->children)>0)?true:false;
    }

    public function getChildren()
    {
	if (!$this->hasDescendants())
	    return false;

	return $this->children;
    }

    public function getParent()
    {
	return (is_object($this->parent))?($this->parent):(false); 
    }

    public function getRoot()
    {
   	$temp = $this;
	while ($temp->getParent())
	    $temp = $temp->getParent();

	return $temp;
    }

    public function getDepth()
    {
	$depth = 1;
   	$temp = $this;
	while ($temp->getParent())
	{
	    $temp = $temp->getParent();
	    $depth++;
	}
	return $depth;
    }

    public function getPath($reverse=false)
    {
	$array = array();

   	$temp = $this;
	$array[]=$temp;
	while ($temp->getParent())
	{
	    $temp = $temp->getParent();
	    $array[]=$temp;
	}
	return ($reverse==false)?(array_reverse($array)):($array);
    }

    public function getNodeById($id,$childNodes=NULL,$step=0,$temp=false)
    {
	if ($this->id == $id)
	    return $this;

	if ($step==0)
	    $childNodes = $this->getChildren();
	
	if ($childNodes)
	{
	    foreach ($childNodes as $child)
	    {
		if ($child->id==$id)
		{
		    $temp = $child;
		    break;
		}
		elseif ($child->hasDescendants())
		{
		    $step++;
		    $this->getNodeById($id,$child->getChildren(),$step,&$temp);
		    $step--;
		}
	    }
	}	
	if ($step==0)
	    return $temp;
    }

    private function rebuild($child=0, $left=1,$step=0)
    {
	$child;
	if ($step==0)
	    $child = $this->getRoot();
	$child->left = $left++;

	if ($child->hasDescendants())
	foreach ($child->getChildren() as $temp) 
	{
	    $step++;
	    $this->rebuild(&$temp, &$left,$step);
	    $step--;
	}
	$child->right = $left++;
    }

    public function move($item,$order=0,$parent=-1)
    {
	if ($this->id!=$this->parent_id)
	    return;

	if ($order<0)
	    return false;

	if (is_numeric($item))
	    $item = $this->getNodeById($item);

	if ($item->id==$item->parent_id)
	    return;

	if (is_numeric($parent))
	{
	    if ($parent==-1)
		$parent = $item->getParent();
	    else
		$parent = $this->getNodeById($parent);
	}
	if ( ($item->getParent() === $parent) && ($order==array_search($item,$parent->children,true)) )
	    return true;

	if ( ($order>=count($parent->children)) && ($order>0) )
	    return false;

	unset($item->getParent()->children[array_search($item,$item->getParent()->children,true)]);
	if ($item->getParent() !== $parent)
	{
	    $item->parent = $parent;
	    $item->parent_id = $parent->id;
	}
	$parent->right++;
	array_splice($parent->children, $order, 0, array($item));
	$this->rebuild();
	$this->save();
	return true;
    }

    public function insertItem($parent=-1)
    {
	if ($this->id!=$this->parent_id)
	    return;

	if (is_numeric($parent))
	    if ($parent==-1)
		$parent = $this->getRoot();
	    else
		$parent = $this->getNodeById($parent);
	
	$class = get_class($this);
	$temp = new $class((object)array());
	$parent->addChild($temp);

	$query = Database::instance()->query('INSERT INTO `'.$this->table.'`(parent_id) VALUES('.$parent->id.')');
	$temp->set($query->insert_id(),false);
	$this->rebuild();
	$this->save();
	return $temp;
    }

    public function removeItem($item)
    {
	if ($this->id!=$this->parent_id)
	    return;

	if (is_numeric($item))
	    $item = $this->getNodeById($item);

	if ($item->id==$item->parent_id)
	    return;

	Database::instance()->query('DELETE FROM `'.$this->table.'` WHERE `left` BETWEEN '.$item->left.' AND '.$item->right);
	
	if (isset($item->getParent()->children[array_search($item,$item->getParent()->children,true)]))
	    unset($item->getParent()->children[array_search($item,$item->getParent()->children,true)]);

	$this->rebuild();
	$this->save();
    }

    public function update($item)
    {
	if ($this->id!=$this->parent_id)
	    return;

	if (is_numeric($item))
	    $item = $this->getNodeById($item);
	$query1 = 'UPDATE `'.$this->table.'` SET ';
	$query2 = '';
	$query3 = ' WHERE `id`='.$item->id;
	foreach ($item->data as $key=>$item)
	{
	    if ($query2!="")
		$query2.=",";
	    if ($key != "id")
		$query2.='`'.$key.'`="'.$item.'"';
	}
	$query = $query1.$query2.$query3;
	Database::instance()->query($query);
	return true;
    }

    private function addChild($child)
    {
	$child->parent = $this;
	$this->children[]=$child;
    }

    private function set($item,$auto_load=false)
    {
	if (is_numeric($item))
	{
	    $nodes = Database::instance()->query('SELECT c.* FROM '.$this->table.' c WHERE c.id='.(int)$item);
	    if (count($nodes)<1)
	    {
		echo "Node doesn't exist!";
		return;
	    }
	    $node = $nodes[0];
	    $this->set($node);	    
	    if ($auto_load)
		$this->load();
	}
	else
	{
	    foreach($item as $key => $value)
		$this->__SET($key,$value);
	}
    }

    private function load()
    {
	$result = Database::instance()->query('SELECT * FROM '.$this->table.' WHERE `id`!='.$this->id.' AND `left` BETWEEN '.$this->left.' AND '.$this->right.' ORDER BY `left` ASC;');
	$parent = $this;

	if (count($result)>0)
	foreach ($result as $key => $row)
	{
	    $class = get_class($this);
	    $temp = new $class($row);
	    
	    if ($parent->id==$temp->parent_id)
		$parent->addChild($temp);
	    else
	    {
		$parent = $this->getNodeById($row->parent_id);
		if (!$parent)
		{
		    echo 'Can\'t fint parent: '.$row->parent_id.'<br/>';
		    return;
		}
		$parent->addChild($temp);
	    }
	}
    }

    private function save($childNodes=NULL,$step=0)
    {
	if ($this->getRoot()->id!=$this->getRoot()->parent_id)
	    return false;

	if ($step==0)
	{
	    $childNodes = $this->getRoot()->getChildren();
	    $this->update($this);
	}

	if ($childNodes)
	{
	    foreach ($childNodes as $child)
	    {
		$this->update($child);

		if ($child->hasDescendants())
		{
		    $step++;
		    $this->save($child->getChildren(),$step);
		    $step--;
		}
	    }
	}
	if ($step==0)
	    return true;
    }
}
?>